   /*=========================================================================
                                                                                
  Program:   crea
  Module:    $RCSfile: creaMessageManager.cxx,v $
  Language:  C++
  Date:      $Date: 2009/02/26 12:04:10 $
  Version:   $Revision: 1.2 $
                                                                                
  Copyright (c) CREATIS (Centre de Recherche et d'Applications en Traitement de
  l'Image). All rights reserved. See Doc/License.txt or
  http://www.creatis.insa-lyon.fr/Public/Gdcm/License.html for details.
                                                                                
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.
                                                                                
=========================================================================*/
/**
 * \file
 * \brief class MessageManager : Manages the messages displayed by crea (code)
 */
#include "creaMessageManager.h"

namespace crea 
{

  //===========================================================================
  MessageManager::MessageManager() 
    : mMaxMessageLength(8),
      mSendToCout(true)
  {
    std::string key;

    key ="all";
    mMessageMap[key] = new MessageType(0,"Minimum level for all kind of messages");
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();

    key ="max";
    mMessageMap[key] = new MessageType(9,"Maximum level for all kind of messages");
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();

    key ="info";
    mMessageMap[key] = new MessageType(1,"Information messages");
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();

    key ="warning";
    mMessageMap[key] = new MessageType(1,"Warning messages");
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();

   /*
    key ="max";
    mMessageLevel[key] = 9;
    mMessageHelp[key] = "Maximum level for all kind of messages";
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();
    key = "error";
    mMessageLevel[key] = 0;
    mMessageHelp[key] = "Error messages";
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();
    key = "warning";
    mMessageLevel[key] = 0;
    mMessageHelp[key] = "Warning messages";
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();
    key = "debug";
    mMessageLevel[key] = 0;
    mMessageHelp[key] = "Debug messages";
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();
    key = "info";
    mMessageLevel[key] = 1;
    mMessageHelp[key] = "Information messages";
    if (mMaxMessageLength<key.length()) mMaxMessageLength = key.length();
    */


   }
  //===========================================================================


  //===========================================================================
  MessageManager::~MessageManager() 
  {
    //      std::cout << "~MessageManager"<<std::endl;
    MessageMapType::iterator i;
    for (i=mMessageMap.begin(); i!=mMessageMap.end(); ++i)
      {
	delete i->second;
      }
  }
  //===========================================================================


  //===========================================================================
  MessageManager* MessageManager::GetInstance() 
  { 
    static MessageManager* m = 0;
    if (!m) m = new MessageManager();
    return m; 
  }
  //===========================================================================

  //===========================================================================
  void MessageManager::RegisterMessageType(const std::string& key, 
					   const std::string& help,
					   unsigned char default_level) 
  {
    GetInstance()->mMessageMap[key] = new MessageType(default_level,help);

    if (GetInstance()->mMaxMessageLength<key.length()) 
      GetInstance()->mMaxMessageLength = key.length();
  }
  //===========================================================================

  //===========================================================================
  void MessageManager::SetMessageLevel(const std::string& key, 
				       unsigned char level) 
  {
    MessageMapType::iterator i;
    i = GetInstance()->mMessageMap.find(key);
    if (i!=GetInstance()->mMessageMap.end()) 
      {
	(*i).second->Level = level;
      }
    else 
      {
	creaWarning("MessageManager::SetMessageLevel : message type=<"
		    <<key<<"> unregistered");
      }
    
  }
  //===========================================================================  
  
  //===========================================================================
  int MessageManager::GetMessageLevel(const std::string& key) 
  {
    int l = GetInstance()->mMessageMap["all"]->Level;
    MessageMapType::iterator i = GetInstance()->mMessageMap.find(key);
    if (i!=GetInstance()->mMessageMap.end()) {
      if ( (*i).second->Level > l ) l = (*i).second->Level;      
    }
    int m = GetInstance()->mMessageMap["max"]->Level;
    if (l>m) l=m;
    return l;
  }
  //===========================================================================

  //===========================================================================
  void MessageManager::SendMessagesToCout(bool v)
  {
    GetInstance()->mSendToCout = v;
  }
  //===========================================================================

  //===========================================================================
  void MessageManager::SendMessage(const std::string& key, const std::string& mess)
  {
    if (GetInstance()->mSendToCout)
      {
	std::cout << mess;
      }
    GetInstance()->mMessageMap[key]->Signal(mess);
  }
  //===========================================================================

  //===========================================================================
  void MessageManager::AddMessageObserver( const std::string& key, MessageCallbackType callback )
  {
    GetInstance()->mMessageMap[key]->Signal.connect(callback);
  }
  //===========================================================================
 
  //===========================================================================
  void MessageManager::PrintInfo() 
  {
    creaMessage("info",1,"================ Messages =================" 
		<< creaendl);
    creaMessage("info",1, "Kind");
    for (int k=0;
	 k<(int)(GetInstance()->mMaxMessageLength-2);
	 k++) 
      {
	creaMessageCont("info",1," "); 
      }
    creaMessageCont("info",1,"Level  Nature" << creaendl);
    MessageMapType::iterator i;
    for (i=GetInstance()->mMessageMap.begin();
	 i!=GetInstance()->mMessageMap.end();
	 ++i) 
      {
	creaMessage("info",1, (*i).first);
	for (int k=0;
	     k<(int)(GetInstance()->mMaxMessageLength+2-(*i).first.length());
	     k++) {
	  creaMessageCont("info",1," ");
	}
	creaMessageCont("info",1, (*i).second->Level << "\t" 
			<< (*i).second->Help << creaendl);
    }
    creaMessage("info",1,"===========================================" 
		<< creaendl);
  }
    //===========================================================================

}
